package cc.fascinated.chat;

import cc.fascinated.account.Account;
import cc.fascinated.config.Lang;
import cc.fascinated.utils.Manager;
import cc.fascinated.utils.MessageUtils;
import cc.fascinated.utils.Style;
import io.papermc.paper.event.player.AsyncChatEvent;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.minimessage.MiniMessage;

import java.util.HashMap;
import java.util.regex.Pattern;

public class ChatManager extends Manager {

    private final HashMap<Account, String> lastMessage = new HashMap<>();
    private final Pattern domainPattern = Pattern.compile("^((?!-)[A-Za-z0-9-]{1,63}(?<!-)\\.)+[A-Za-z]{2,8}$");
    private final Pattern ipPattern = Pattern.compile("\\b(?:\\d{1,3}\\.){3}\\d{1,3}\\b");

    @Override
    public void onAsyncChat(Account account, AsyncChatEvent event) {
        NamedTextColor color = account.getPlayerColorProfile().getColor();
        BlockReason blockReason = null;

        TextComponent textComponent = (TextComponent) event.message();
        String messageContent = textComponent.content();

        // Check if the message contains a domain and is not the server's domain.
        if (domainPattern.matcher(messageContent).find() && !messageContent.contains("aetheria.cc")) {
            blockReason = BlockReason.DOMAIN;
        }

        // Check if the message contains an IP address.
        if (ipPattern.matcher(messageContent).find()) {
            blockReason = BlockReason.IP;
        }

        // Block the message if it is the same as the last message sent.
        if (lastMessage.containsKey(account) && lastMessage.get(account).equalsIgnoreCase(messageContent)) {
            blockReason = BlockReason.DUPLICATE;
        }

        if (blockReason != null) {
            event.setCancelled(true);
            account.sendMessage(Lang.BLOCKED_MESSAGE.getAsString()
                    .replace("%reason%", blockReason.getReason())
            );

            if (blockReason != BlockReason.DUPLICATE) {
                MessageUtils.messageOps(Lang.BLOCKED_MESSAGE_ALERT.getAsString()
                        .replace("%player%", account.getName())
                        .replace("%message%", messageContent)
                );
            }
            return;
        }

        event.renderer((source, sourceDisplayName, message, viewer) -> {
            MiniMessage miniMessage = Style.getMiniMessage();
            return miniMessage.deserialize(Lang.CHAT_FORMAT.getAsString()
                    .replace("%chatcolor%", color.toString())
                    .replace("%name%", account.getName())
                    .replace("%message%", miniMessage.stripTags(messageContent))
            );
        });
        lastMessage.put(account, messageContent);
    }

    @RequiredArgsConstructor
    @Getter
    private enum BlockReason {
        DOMAIN("Domain was detected in the message."),
        DUPLICATE("Duplicate message was detected."),
        IP("IP address was detected in the message.");

        private final String reason;
    }
}
