package cc.fascinated.command;

import cc.fascinated.account.Account;
import cc.fascinated.account.AccountManager;
import lombok.Getter;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

@Getter
public abstract class Command extends org.bukkit.command.Command {

    public Command(@NotNull String name) {
        super(name);
    }

    /**
     * Executes the command.
     *
     * @param account The account executing the command.
     * @param args    The arguments of the command.
     */
    public abstract void execute(Account account, String[] args);

    /**
     * Tab completes the command.
     *
     * @param account The account executing the command.
     * @param args    The arguments of the command.
     * @return The tab completions.
     */
    public List<String> tabComplete(Account account, String[] args) {
        return null;
    }

    @Override
    public final boolean execute(@NotNull CommandSender commandSender, @NotNull String s, @NotNull String[] strings) {
        Player player = (Player) commandSender;
        Account account = AccountManager.getAccount(player.getUniqueId());

        this.execute(account, strings);
        return true;
    }

    @Override
    public final @NotNull List<String> tabComplete(@NotNull CommandSender sender, @NotNull String alias, @NotNull String[] args) throws IllegalArgumentException {
        Player player = (Player) sender;
        Account account = AccountManager.getAccount(player.getUniqueId());
        return this.tabComplete(account, args);
    }
}
