package cc.fascinated.event;

import cc.fascinated.Aetheria;
import cc.fascinated.account.Account;
import cc.fascinated.account.AccountManager;
import com.vexsoftware.votifier.model.VotifierEvent;
import io.papermc.paper.event.player.AsyncChatEvent;
import lombok.Getter;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.EntityDeathEvent;
import org.bukkit.event.player.PlayerCommandPreprocessEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerLoginEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.event.server.PluginDisableEvent;
import org.bukkit.event.server.ServerListPingEvent;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

public class EventManager implements Listener {

    /**
     * All registered listeners.
     */
    @Getter
    private static final List<EventListener> LISTENERS = Collections.synchronizedList(new LinkedList<>());

    public EventManager() {
        Aetheria.INSTANCE.getServer().getPluginManager().registerEvents(this, Aetheria.INSTANCE);
    }

    /**
     * Registers a new listener.
     *
     * @param listener the listener to register
     */
    public static void registerListener(EventListener listener) {
        LISTENERS.add(listener);
        LISTENERS.sort(Comparator.comparingInt(registeredListener -> registeredListener.getPriority().ordinal()));
    }

    @EventHandler
    public void onPluginDisable(PluginDisableEvent event) {
        if (event.getPlugin() != Aetheria.INSTANCE) {
            return;
        }
        for (EventListener listener : LISTENERS) {
            listener.onAetheriaDisable();
        }
    }

    @EventHandler
    public void onPlayerJoin(PlayerJoinEvent event) {
        Player player = event.getPlayer();
        Account account = AccountManager.getAccount(player.getUniqueId());

        for (EventListener listener : LISTENERS) {
            listener.onPlayerJoin(account, event);
        }
    }

    @EventHandler
    public void onPlayerLogin(PlayerLoginEvent event) {
        Player player = event.getPlayer();
        Account account = AccountManager.getAccount(player.getUniqueId());

        for (EventListener listener : LISTENERS) {
            listener.onPlayerLogin(account, event);
        }
    }

    @EventHandler
    public void onPlayerQuit(PlayerQuitEvent event) {
        Player player = event.getPlayer();
        Account account = AccountManager.getAccount(player.getUniqueId());

        for (EventListener listener : LISTENERS) {
            listener.onPlayerQuit(account, event);
        }
    }

    @EventHandler
    public void onAsyncChat(AsyncChatEvent event) {
        Player player = event.getPlayer();
        Account account = AccountManager.getAccount(player.getUniqueId());

        for (EventListener listener : LISTENERS) {
            listener.onAsyncChat(account, event);
        }
    }

    @EventHandler
    public void onCommandPreProcess(PlayerCommandPreprocessEvent event) {
        Player player = event.getPlayer();
        Account account = AccountManager.getAccount(player.getUniqueId());

        for (EventListener listener : LISTENERS) {
            listener.onCommandPreProcess(account, event);
        }
    }

    @EventHandler
    public void onServerListPing(ServerListPingEvent event) {
        for (EventListener listener : LISTENERS) {
            listener.onServerListPing(event);
        }
    }

    @EventHandler
    public void onEntityDeath(EntityDeathEvent event) {
        for (EventListener listener : LISTENERS) {
            listener.onEntityDeath(event);
        }
    }

    @EventHandler
    public void onVote(VotifierEvent event) {
        Account account = AccountManager.getAccount(Bukkit.getOfflinePlayer(event.getVote().getUsername()).getUniqueId());
        if (account == null) {
            return;
        }

        for (EventListener listener : LISTENERS) {
            listener.onVote(account, event);
        }
    }
}
