package cc.fascinated.playercolor;

import cc.fascinated.account.Account;
import cc.fascinated.playercolor.command.PlayerColorCommand;
import cc.fascinated.utils.Manager;
import lombok.Getter;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.scoreboard.Scoreboard;
import org.bukkit.scoreboard.Team;

import java.util.List;

public class PlayerColorManager extends Manager {

    /**
     * List of possible colors that can be randomly picked
     */
    @Getter
    private static final List<NamedTextColor> validColors = List.of(
            NamedTextColor.DARK_BLUE,
            NamedTextColor.DARK_GREEN,
            NamedTextColor.DARK_AQUA,
            NamedTextColor.DARK_RED,
            NamedTextColor.DARK_PURPLE,
            NamedTextColor.GOLD,
            NamedTextColor.GRAY,
            NamedTextColor.DARK_GRAY,
            NamedTextColor.BLUE,
            NamedTextColor.GREEN,
            NamedTextColor.AQUA,
            NamedTextColor.RED,
            NamedTextColor.LIGHT_PURPLE,
            NamedTextColor.YELLOW,
            NamedTextColor.WHITE
    );

    public PlayerColorManager() {
        registerCommand(new PlayerColorCommand());
    }

    /**
     * Gets a random color from the list of valid colors.
     *
     * @return the random color
     */
    public static NamedTextColor getRandomColor() {
        return validColors.get((int) (Math.random() * validColors.size()));
    }

    /**
     * Gets the text color from a string.
     *
     * @param color the color string
     * @return the text color
     */
    public static NamedTextColor getColor(String color) {
        for (NamedTextColor textColor : validColors) {
            if (textColor.toString().equalsIgnoreCase(color)) {
                return textColor;
            }
        }

        return null;
    }

    /**
     * Gets the scoreboard team for this color.
     *
     * @param color the color to get the team for
     * @return the team
     */
    public static Team getScoreboardTeam(NamedTextColor color) {
        Scoreboard scoreboard = Bukkit.getScoreboardManager().getMainScoreboard();
        Team team = scoreboard.getTeam(color.toString().toLowerCase());
        if (team == null) {
            team = scoreboard.registerNewTeam(color.toString().toLowerCase());
            team.color(color);
        }

        return team;
    }

    @Override
    public void onPlayerJoin(Account account, PlayerJoinEvent event) {
        PlayerColorProfile playerColor = account.getPlayerColorProfile();

        Team team = PlayerColorManager.getScoreboardTeam(playerColor.getColor());
        team.addEntry(account.getName());
    }

    @Override
    public void onPlayerQuit(Account account, PlayerQuitEvent event) {
        PlayerColorProfile playerColor = account.getPlayerColorProfile();

        Team team = PlayerColorManager.getScoreboardTeam(playerColor.getColor());
        team.removeEntry(account.getName());
    }
}
