package cc.fascinated.playercolor;

import cc.fascinated.account.Account;
import cc.fascinated.account.Profile;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.scoreboard.Team;

@Getter
@Setter
@EqualsAndHashCode(callSuper = false)
public class PlayerColorProfile extends Profile {

    /**
     * The current color of the player.
     */
    private NamedTextColor color;

    public PlayerColorProfile(Account account, ConfigurationSection section) {
        super(account, section);

        if (section == null) {
            this.color = PlayerColorManager.getRandomColor();
        } else {
            this.color = NamedTextColor.namedColor(section.getInt("color"));
        }
    }

    /**
     * Sets the color of the player.
     *
     * @param color the new color
     */
    public void setColor(NamedTextColor color) {
        this.color = color;

        Team team = PlayerColorManager.getScoreboardTeam(this.color);
        team.addEntry(getAccount().getName());
    }

    @Override
    public void save(ConfigurationSection section) {
        section.set("color", this.color.value());
    }
}
