package cc.fascinated.playercolor.command;

import cc.fascinated.account.Account;
import cc.fascinated.command.Command;
import cc.fascinated.config.Lang;
import cc.fascinated.playercolor.PlayerColorManager;
import cc.fascinated.playercolor.PlayerColorProfile;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;

import java.util.ArrayList;
import java.util.List;

public class PlayerColorCommand extends Command {

    public PlayerColorCommand() {
        super("playercolor");
    }

    @Override
    public void execute(Account account, String[] args) {
        List<NamedTextColor> validColors = PlayerColorManager.getValidColors();
        PlayerColorProfile playerColorProfile = account.getPlayerColorProfile();

        if (args.length == 0) {
            account.sendMessage(Lang.PLAYER_COLOR_COMMAND_USAGE.getAsString());

            Component colors = Component.text("Colors: ");
            for (NamedTextColor color : validColors) {
                colors = colors.append(Component.text(color.toString()).color(color)).append(Component.text(", "));
            }
            colors = colors.append(Component.text("random"));
            account.sendMessage(colors);
            return;
        }

        NamedTextColor color = args[0].equalsIgnoreCase("random") ?
                PlayerColorManager.getRandomColor() : PlayerColorManager.getColor(args[0]);
        if (color == null) {
            account.sendMessage(Lang.PLAYER_COLOR_COMMAND_INVALID_COLOR.getAsString());
            return;
        }

        playerColorProfile.setColor(color);
        account.sendMessage(Lang.PLAYER_COLOR_COMMAND_COLOR_CHANGED.getAsString()
                .replace("%color%", playerColorProfile.getColor().toString())
                .replace("new-color", playerColorProfile.getColor().toString())
        );
    }

    @Override
    public List<String> tabComplete(Account account, String[] args) {
        List<NamedTextColor> colors = PlayerColorManager.getValidColors();

        if (args.length == 1) {
            List<String> list = new ArrayList<>(colors.stream().map(NamedTextColor::toString).toList());
            list.add("random");
            return list;
        }
        return null;
    }
}
