package cc.fascinated.utils;

import cc.fascinated.config.Lang;
import lombok.Getter;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.minimessage.MiniMessage;
import net.kyori.adventure.text.minimessage.tag.Tag;
import net.kyori.adventure.text.minimessage.tag.resolver.TagResolver;
import net.kyori.adventure.text.minimessage.tag.standard.StandardTags;

import java.util.ArrayList;
import java.util.List;

public class Style {

    @Getter
    private static final MiniMessage miniMessage;

    static {
        List<TagResolver> tagResolvers = new ArrayList<>();
        tagResolvers.add(TagResolver.resolver("prefix", (context, argumentQueue) -> {
            return Tag.inserting(MiniMessage.miniMessage().deserialize(Lang.PREFIX.getAsString()));
        }));
        tagResolvers.add(TagResolver.resolver("admin-prefix", (context, argumentQueue) -> {
            return Tag.inserting(MiniMessage.miniMessage().deserialize(Lang.ADMIN_PREFIX.getAsString()));
        }));

        miniMessage = MiniMessage.builder()
                .tags(TagResolver.builder()
                        .resolver(StandardTags.defaults())
                        .resolvers(tagResolvers)
                        .build()
                ).build();
    }

    /**
     * Get the prefix for a class.
     *
     * @param clazz the class
     * @return the prefix
     */
    public static Component getPrefix(Class<?> clazz) {
        String className = clazz.getSimpleName()
                .replace("manager", "") // PlayerColorManager -> PlayerColor
                .replace("command", "") // PlayerColorCommand -> PlayerColor
                .toUpperCase();
        return miniMessage.deserialize(Lang.PREFIX_FORMAT.getAsString()
                .replace("%class%", className)
        );
    }
}
