package cc.fascinated.worldsize;

import cc.fascinated.Aetheria;
import cc.fascinated.command.CommandManager;
import cc.fascinated.placeholder.PlaceholderManager;
import cc.fascinated.worldsize.command.WorldSizeCommand;
import lombok.Getter;
import org.bukkit.Bukkit;
import org.bukkit.World;
import org.codehaus.plexus.util.FileUtils;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

public class WorldSizeManager {

    private static final int INTERVAL = 10; // in minutes

    /**
     * The total size of all worlds.
     */
    @Getter
    private static final Map<World, Long> worldSizes = new HashMap<>();

    /**
     * The last time the total world size was updated.
     */
    @Getter
    private static long lastUpdated;

    public WorldSizeManager() {
        CommandManager.registerCommand(new WorldSizeCommand());
        PlaceholderManager.registerPlaceholder(new WorldSizePlaceholder());

        Aetheria.INSTANCE.getServer().getAsyncScheduler().runAtFixedRate(Aetheria.INSTANCE, (task) -> {
            calculateTotalWorldSize();
        }, 0, INTERVAL, TimeUnit.MINUTES);
    }

    /**
     * Calculate the total size of all worlds.
     */
    public void calculateTotalWorldSize() {
        for (World world : Bukkit.getWorlds()) {
            File worldFolder = world.getWorldFolder();

            long size = FileUtils.sizeOfDirectory(worldFolder);
            worldSizes.put(world, size);
        }
        lastUpdated = System.currentTimeMillis();
    }
}