package cc.fascinated.worldsize.command;

import cc.fascinated.account.Account;
import cc.fascinated.command.Command;
import cc.fascinated.config.Lang;
import cc.fascinated.utils.FormatterUtils;
import cc.fascinated.utils.TimeUtils;
import cc.fascinated.worldsize.WorldSizeManager;
import org.bukkit.World;

import java.util.Map;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Stream;

public class WorldSizeCommand extends Command {

    public WorldSizeCommand() {
        super("worldsize");
    }

    @Override
    public void execute(Account account, String[] args) {
        account.sendMessage(Lang.WORLD_SIZE_COMMAND_HEADER.getAsString());
        AtomicLong totalSize = new AtomicLong();
        Stream<Map.Entry<World, Long>> sorted = WorldSizeManager.getWorldSizes().entrySet().stream()
                .sorted((o1, o2) -> Long.compare(o2.getValue(), o1.getValue()));
        sorted.forEach((entry) -> {
            account.sendMessage(Lang.WORLD_SIZE_COMMAND_FORMAT.getAsString()
                    .replace("%world%", entry.getKey().getName())
                    .replace("%size%", FormatterUtils.formatBytes(entry.getValue()))
            );
            totalSize.addAndGet(entry.getValue());
        });

        for (String line : Lang.WORLD_SIZE_COMMAND_FOOTER.getAsStringList()) {
            account.sendMessage(line
                    .replace("%total%", FormatterUtils.formatBytes(totalSize.get()))
                    .replace("%time%", TimeUtils.format(System.currentTimeMillis() - WorldSizeManager.getLastUpdated()))
            );
        }
    }
}
